--[[
	Copyright (C) 2017  Fuhrbolg

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>

	Author's note: Under no circumstances should this Addon be used under any situation.
	It is dangerous and will cause significant harm to your computer. You have been warned.
--]]

--[[
	MongoMon is an addon that has the following functionality:
	
		1) Updates UnitFrame portraits with a new portrait. All images are in .tga format
		and are located in the Res folder
		2) Plays a sound when player is awarded killing blow. Sound files are .mp3 format
		and are also located in the Res folder.
		3) Creates and displays a frame with a personalized scoreboard. This frame is draggable.
		If the player is ranked in the top 5 in killing blows a special icon will be displayed
		on the scoreboard.
		4) Nameplates will display Kill - Death score above players' names.
		5) Players ranked in the top 5 in killing blows will get a special icon above their
		nameplate as well.
		
	This addon uses the Ace3 framework. This is used to accomplish the following:
	
		1) Abstract persistent storage using AceDB.
		2) Generate the Options Interface page that is part of blizzard's UI.
		3) Interationalization support for multiple languages. All of these translation
		files are located in the Locale folder.
		4) Console support. This is trivial but meh, when in mexico.
		5) Im not sure if LibStub is part of the Ace3 framework but that is used to load
		libraries.
		6) LibNameplatesRegistry, again not sure if this part of Ace3. It is used to get
		access to player nameplates.
		7) LibButtonGlow, this library is used to make the scoreboard flash. Im way too
		fat and lazy to do that myself. The flash occurs after a killing blow.
		
	Known Issues:
	
		1) Responsiveness. The personalized scoreboard and nameplates update sporadically. 
		MongoMon hooks into COMBAT_LOG_EVENT_UNFILTERED and requests a battle ground score
		update when the sub event is UNIT_DIED. It also requests a battleground score
		when the player changes options and finally when out of combat which is
		the PLAYER_HEALTH_REGEN event.
		
	I photoshopped the majority of the images myself. They are all terrible. Except the
	one with the mad baby which is my nephew. Very cuddly. But also very, very mad
	at all times. The sounds are from Return to Castle Wolfenstein. Whats better than
	a nice german soldat praising you? Nothing. On that subject there is an excellent
	book written by Col. Hans Von Luck, who was a Panzer Commander in the Wehrmacht.
	True gentleman. And another book called "A Higher Call" about a Luftwaffe ace named
	Frans Stiegler. 
	
	Frans saw a banged up b17 flying low over the coast of France and instead of pwning
	it in his 109 he escorted it over German flak batteries so they wouldn't shoot it.
	The american b17 pilot charlie brown (heh) eventually made it back to england. In
	the early 2000's Charlie Brown posted an article in an aviation magazine talking
	about his strange encounter with the friendly Me109 pilot. Frans saw the article
	and they met in Florida.
		
--]]

local ADDON_NAME, T = ...

--[[
 -	Library verification follows.
--]]
if not LibStub then
    error(ADDON_NAME, " requires LibStub")
    return
end

if not LibStub("AceAddon-3.0") then
    error(ADDON_NAME, " requires AceAddon-3.0")
    return;
end

if not LibStub("AceGUI-3.0") then
    error(ADDON_NAME, " requires AceGUI-3.0")
    return;
end

if not LibStub("AceLocale-3.0") then
    error(ADDON_NAME, " requires AceLocale-3.0")
    return
end

if not LibStub("CallbackHandler-1.0") then
    error(ADDON_NAME, " requires CallbackHandler-1.0")
    return
end

if not LibStub("LibNameplateRegistry-1.0") then
    error(ADDON_NAME, " requires LibNameplateRegistry-1.0")
    return
end

if not LibStub("LibButtonGlow-1.0") then
	error(ADDON_NAME, " requires LibButtonGlow-1.0")
	return
end

if not LibStub("AceConfig-3.0") then
	error(ADDON_NAME, " requires AceConfig-3.0")
	return
end

if not LibStub("AceDB-3.0") then
	error(ADDON_NAME, " requires AceDB-3.0")
	return
end

if not LibStub("AceConsole-3.0") then
	error(ADDON_NAME, " requires AceConsole-3.0")
	return
end

-- Localization
local L = LibStub("AceLocale-3.0"):GetLocale("MongoMon")

-- Our addon Table
MongoMon = {

	-- Our options database. Here we store all the flags for toggling sounds, nameplates, portraits, etc.
	db = {},

	-- All players in BG { "<PlayerName>-<ServerName>" = { "portrait" = "<PortraitIcon>", ... }, ...}
	players = {},
	
	-- Array of Nameplates, keyed on player name { "<PlayerName>-<ServerName>" = { "fontString" = "<FontString>", ... }, ...}
	plates = {},
	
	-- Array of player name, keyed on guid { "<GUID>" = "<PlayerName>", ... }
	guids = {},
	
	-- Ref to our particular player data structure
	player = nil,
	
	-- Boolean indicating we are in instanced PvP
	instancedPvp = false,
	
	-- Player's faction name
	PLAYER_FACTION = "",
	
	-- Player's name
	PLAYER_NAME = "",
	
	-- Pet name fucking hunters
	PET_NAME = "",
	
	-- Only reward mongos whose rank is below this. We currently only have 5 portraits so.
	MAX_RANK = 5,
	
	-- Array of our sound files, played when target dies, praise be unto Hillary
	SOUND_FILES = {
		"Interface\\AddOns\\MongoMon\\Res\\Wunderbar.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\GoodGame.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\Sorry.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\Excellent.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\Jahahaha.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\AufWiedersehen.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\EnemyWeakened.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\GreatShot.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\JaWohl.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\AllClear.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\PathCleared.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\Halo.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\Nein.mp3",
		"Interface\\AddOns\\MongoMon\\Res\\JaWohl2.mp3"
	},
	
	-- Array of our terrible portraits
	IMAGE_FILES = {
		["Rogue"] = {
			["Assassination"] = "Interface\\AddOns\\MongoMon\\Res\\Broseph",
			["Outlaw"] = "Interface\\AddOns\\MongoMon\\Res\\Broseph",
			["Subtlety"] = "Interface\\AddOns\\MongoMon\\Res\\Broseph"
		},
		["Death Knight"] = {
			["Blood"] = "Interface\\AddOns\\MongoMon\\Res\\Emo",
			["Frost"] = "Interface\\AddOns\\MongoMon\\Res\\Emo",
			["Unholy"] = "Interface\\AddOns\\MongoMon\\Res\\Emo"
		},
		["Warrior"] = {
			["Fury"] = "Interface\\AddOns\\MongoMon\\Res\\MadBaby",
			["Arms"] = "Interface\\AddOns\\MongoMon\\Res\\MadBaby",
			["Protection"] = "Interface\\AddOns\\MongoMon\\Res\\MadBaby"
		},
		["Hunter"] = {
			["Survival"] = "Interface\\AddOns\\MongoMon\\Res\\DerpFace",
			["Beast Mastery"] = "Interface\\AddOns\\MongoMon\\Res\\DerpFace",
			["Marksmanship"] = "Interface\\AddOns\\MongoMon\\Res\\DerpFace"
		},
		["Priest"] = {
			["Shadow"] = "Interface\\AddOns\\MongoMon\\Res\\Kitten"
		},
		["Paladin"] = {
			["Retribution"] = "Interface\\AddOns\\MongoMon\\Res\\AngryGirl"
		},
		["Druid"] = {
			["Restoration"] = "Interface\\AddOns\\MongoMon\\Res\\CatNerd",
			["Feral"] = "Interface\\AddOns\\MongoMon\\Res\\CatLady"
		},
		["Warlock"] = {
			["Affliction"] = "Interface\\AddOns\\MongoMon\\Res\\Impossibru",
			["Demonology"] = "Interface\\AddOns\\MongoMon\\Res\\Impossibru",
			["Destruction"] = "Interface\\AddOns\\MongoMon\\Res\\Impossibru"
		}
	}
}

-- Inject Ace3 Framework support
LibStub("AceAddon-3.0"):NewAddon(MongoMon, "MongoMon", "LibNameplateRegistry-1.0", "AceConsole-3.0")

-- Slash command /mm opens the options interface and prints out the version.
MongoMon:RegisterChatCommand("mm", "ConsoleCommand")
function MongoMon:ConsoleCommand(input)
	InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
	InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
	print(L["Console"] .. " Version " .. GetAddOnMetadata("MongoMon", "Version") .. ".")
end

--[[
 -	Options table follows. This defines the options that will be dispalyed on
 -	the Interface Options page within the Blizzard Addons menu.
--]]

-- Our AceDB default options
local defaults = {
	profile = {
		portraitEnabled = true,
		scoreEnabled = true,
		nameplatesEnabled = true,
		flashEnabled = true,
		soundEnabled = true
	}
}

-- Our AceDB options table
local options = {
	type = "group",
	name = "MongoMon",
	desc = L["Description"],
	handler = MongoMon,
	set = "OptionSetterHandler",
	get = "OptionGetterHandler",
	args = {
		optionsSubtitle = {
			order = 1,
			type = "header",
			name = L["Subtitle"]
		},
		optionsDescription = {
			order = 2,
			type = "description",
			name = L["Description"]
		},
		portraitTitle = {
			order = 3,
			type = "header",
			name = ""
		},
		portraitEnabled = {
			order = 4,
			type = "toggle",
			name = L["EnablePortrait"]
		},
		portraitDescription = {
			order = 5,
			type = "description",
			name = L["PortraitDescription"]
		},
		scoreTitle = {
			order = 6,
			type = "header",
			name = ""
		},
		scoreEnabled = {
			order = 7,
			type = "toggle",
			name = L["EnableScore"]
		},
		scoreDescription = {
			order = 8,
			type = "description",
			name = L["ScoreDescription"]
		},
		nameplatesTitle = {
			order = 9,
			type = "header",
			name = ""
		},
		nameplatesEnabled = {
			order = 10,
			type = "toggle",
			name = L["EnableNameplates"],
		},
		nameplatesDescription = {
			order = 11,
			type = "description",
			name = L["NameplatesDescription"]
		},
		flashTitle = {
			order = 12,
			type = "header",
			name = ""
		},
		flashEnabled = {
			order = 13,
			type = "toggle",
			name = L["EnableFlash"]
		},
		flashDescription = {
			order = 14,
			type = "description",
			name = L["FlashDescription"]
		},
		soundTitle = {
			order = 15,
			type = "header",
			name = ""
		},
		soundEnabled = {
			order = 16,
			type = "toggle",
			name = L["EnableSound"]
		},
		soundDescription = {
			order = 17,
			type = "description",
			name = L["SoundDescription"]
		}
	}
}

-- These 4 sub events can contain an overkill amount.
local damageEvent = { ["SWING_DAMAGE"] = true, ["RANGE_DAMAGE"] = true, ["SPELL_DAMAGE"] = true, ["SPELL_PERIODIC_DAMAGE"] = true }
 
-- So everything in this game that is event related must be part of a widget.
local frame = CreateFrame("FRAME", "MongoMonFrame")

-- Scoreboard frame that will display Kill - Death in BG's
local score = CreateFrame("FRAME", "MongoMonScoreFrame")
score:SetMovable(true)
score:EnableMouse(true)
score:RegisterForDrag("LeftButton")
score:SetScript("OnDragStart", score.StartMoving)
score:SetScript("OnDragStop", score.StopMovingOrSizing)
score:SetPoint("CENTER"); 
score:SetWidth(224); 
score:SetHeight(64);
score:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
	tile = true, tileSize = 16, edgeSize = 16, 
	insets = { left = 4, right = 4, top = 4, bottom = 4 }
});
score:SetBackdropColor(1.0, 0.5, 0.5, 0.3);

-- Title 
score.title = score:CreateFontString(nil, "ARTWORK", "GameFontHighlightOutline")
score.title:SetPoint("BOTTOM", score, "TOP", 0, -5) -- Places frame in center of screen
score.title:SetText(L["Title"])

-- Fontstring
score.fontString = score:CreateFontString(nil, "ARTWORK", "GameFontNormalHugeOutline2")
score.fontString:SetPoint("CENTER", score) -- Places frame in center of screen

-- High Rank Crest Texture
score.crestFrame = CreateFrame("FRAME", nil, score)
score.crestFrame:SetFrameLevel(127) -- For issue #1, the frame layer was spastic. Occasionally displayed behind the rank crest texture.
score.crestFrame:SetWidth(60)
score.crestFrame:SetHeight(60)
score.crestFrame:SetPoint("BOTTOMLEFT", score, "TOPLEFT", -3, -10)
score.crestTex = score.crestFrame:CreateTexture()
score.crestTex:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrest")
score.crestTex:SetAllPoints()

-- High Rank Texture
score.rankFrame = CreateFrame("FRAME", nil, score.crestFrame)
score.rankFrame:SetFrameLevel(128) -- For issue #1, the frame layer was spastic. Occasionally displayed behind the rank crest texture.
score.rankFrame:SetWidth(40)
score.rankFrame:SetHeight(40)
score.rankFrame:SetPoint("CENTER", score.crestFrame)
score.rankTex = score.rankFrame:CreateTexture()
score.rankTex:SetAllPoints()

--[[
 -	Utility function used for sorting.
 -
 - An ordered sort. This is exceptionally handy and is used to order
 - the battleground participants by killing blows then defaults to 
 - lower number of deaths in the event of a tie. This particular
 - function is a generic sort. The man who wrote this must be jesus
 - because I have no idea what's going on. It just works. Praise Hillary.
 -
 - http://stackoverflow.com/questions/15706270/sort-a-table-in-lua
--]]
local function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

--[[
 -	This is an event handler for the Ace3 framework. When the addon is first loaded
 -	it will fire. We are just going to use it to handle the loading of our 
 -	options database.
--]]
function MongoMon:OnInitialize() 
	-- Generate the database of stored values
	self.db = LibStub("AceDB-3.0"):New("MongoMonDB", defaults, true).profile
	-- Register the options. This library handles all of the magic.
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MongoMon", options)
	-- Generate the Options Interface
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MongoMon", "MongoMon")
end

--[[
 -	Callback when an option is set in the Interface Options menu.
 -	
 -	Use this to update our internal storage.
--]]
function MongoMon:OptionSetterHandler(info, value)
	self.db[info[#info]] = value
	if self.instancedPvp then -- Re-initialize state if user changes settings during a battleground
		self:Initialize()
	end
end

--[[
 -	Callback when to get the value of an options item.
--]]
function MongoMon:OptionGetterHandler(info)
	return self.db[info[#info]]
end

--[[
 -	Prepares the Addon by registering the required events
 -	when a player enters a zone or logs into the game world.
 -	
 -	Has the reverse effect when player zones out of PvP by unregistering
 -	events.
 -	
 -	Note: We can include the UPDATE_WORLD_STATE event to queue the
 -	server for BG data if need be.
 -	
 -	TODO: Disable for arenas?
 -	isArena, isRegistered = IsActiveBattlefieldArena()
--]]
function MongoMon:Initialize()

	-- Set the instanced PvP flag
	local _, instanceType = IsInInstance()
	self.instancedPvp = instanceType == "pvp"
	
	-- Clears all refs
	table.wipe(self.players)
	table.wipe(self.plates)
	table.wipe(self.guids)
	self.player = nil
	
	self.PLAYER_NAME = UnitName("player")
	self.PET_NAME = UnitName("pet")
	self.PLAYER_FACTION = UnitFactionGroup("player")

	if self.instancedPvp then -- Register events
	
		frame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE") 	-- Portraits, Nameplates
		frame:RegisterEvent("PLAYER_REGEN_ENABLED")			-- Portraits, Nameplates
		frame:RegisterEvent("PLAYER_TARGET_CHANGED")		-- Portraits
		frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")	-- Sounds, Guids, Killing Blows
		
		-- Lib Nameplate Registry callbacks
		self:LNR_RegisterCallback("LNR_ON_NEW_PLATE")		-- Nameplates
		self:LNR_RegisterCallback("LNR_ON_RECYCLE_PLATE")	-- Nameplates
		self:LNR_RegisterCallback("LNR_ON_GUID_FOUND")		-- Nameplates
		
		if self.db.scoreEnabled then
			score.fontString:SetText("0 - 0")
			score:Show()
		end
		
	else -- Events are no longer needed lets be nice and clean them up
		frame:UnregisterEvent("UPDATE_BATTLEFIELD_SCORE")
		frame:UnregisterEvent("PLAYER_REGEN_ENABLED")
		frame:UnregisterEvent("PLAYER_TARGET_CHANGED")
		frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:LNR_UnregisterAllCallbacks()
		score:Hide()
	end
end

--[[
 -	Up until recently I was using a few terrible techniques to determine killing blows 
 -	until I found this here solution http://forums.wowace.com/showthread.php?t=19784.
 -	
 -	It seems like the PARTY_KILL event is unreliable so we have to scan combat logs and
 -	check all player damage for an overkill amount. Seems crappy but here we are.
 -	
 -	NOTE: Read about destFlags (UnitFlag) here: http://www.wowwiki.com/UnitFlag
--]]
function MongoMon:KillingBlowCheck(subEvent, sourceName, destName, destFlags, ...) 

	if not self.db.soundEnabled then return	end	
	if not self.instancedPvp then return end
	if destFlags ~= 66888 then return end

	local overkill
	if subevent == "SWING_DAMAGE" then
		_, overkill = ...
	else
		_, _, _, _, overkill = ...
	end
	
	if overkill and overkill > 1 and (sourceName == self.PLAYER_NAME or sourceName == self.PET_NAME) then
		-- local index = math.random(1, 11)
		local index = math.random(1, table.getn(self.SOUND_FILES))
		local soundFile = self.SOUND_FILES[index]
		PlaySoundFile(soundFile, "master")
	end
end

--[[
 -	Handles PLAYER_TARGET_CHANGED events.
--]]
function MongoMon:PlayerTargetChanged()

	if not self.db.portraitEnabled then return	end	
	if not self.instancedPvp then return end

	if not UnitIsFriend("player", "target") then -- Is target hostile
		if not UnitIsDead("target") then -- Is unit alive

			local name = self:getTargetName()
			
			if not name then return end
			
			local portrait = (self.players[name] and self.players[name].portrait) and self.players[name].portrait or nil
			
			if portrait then -- If nil then it is not overpowered
				TargetFrame.portrait:SetTexture(portrait)
			end
		end
	end	
end

--[[
 -	This is a response from the server to our request battleground score
 -	API call. We are going to cache this data in the MongoMon table.
 -
 -	Here we will set or update our players array and do some calculation to
 -	that determines ranking in the BG and whether the players qualify for
 -	a special nameplate.
--]]
function MongoMon:UpdateBattlefieldScore()

	-- Count of BG participants
	local numBgPlayers = GetNumBattlefieldScores()
	local originalPlayerKills = 0
	if self.player then
		originalPlayerKills = self.player.kills
	end
	for x = 1, numBgPlayers, 1 do 
		name, killingBlows, honorableKills, deaths, honorGained, faction, race, class, classToken, damageDone, healingDone, bgRating, ratingChange, preMatchMMR, mmrChange, talentSpec = GetBattlefieldScore(x)

		-- Around 6.0 blizzard started sending nil combatants into bgs, this is very rng based.
		-- Look around, you'll see the scoreboard has empty player names. GG.
		if name then 
			local player, plate = self.players[name], self.plates[name]
			
			if player then
				player.kills = killingBlows
				player.deaths = deaths
			else
				faction = faction == 0 and "Horde" or "Alliance"
				player = {
					name = name,
					kills = honorableKills,
					deaths = deaths,
					class = class,
					spec = talentSpec,
					portrait = (self.IMAGE_FILES and self.IMAGE_FILES[class] and self.IMAGE_FILES[class][talentSpec]) and self.IMAGE_FILES[class][talentSpec] or nil,
					enemy = faction ~= self.PLAYER_FACTION,
					rank = 0
				}				
			end
			
			-- name-server
			self.players[name] = player 
		end
	end
	
	-- Compute top ranked players in BG based purely on killing blows
	local topRanks = 0 -- Keep assigning rank until we are all done
	for _, player in spairs(self.players, function(t,a,b) 
		if t[b].kills == t[a].kills then return t[b].deaths > t[a].deaths -- Tie goes to player with less deaths
		else return t[b].kills < t[a].kills end
	end) do
	
		topRanks = topRanks + 1
		player.rank = topRanks

		-- Finally update nameplates for those people within range, this includes the changes from above
		-- Update score if it has changed for someone in range, using IsShown to determine if this crap is in range
		local plate = self.plates[name]
		-- if plate and plate.fontString:IsShown() and (honorableKills ~= player.kills or deaths ~= player.deaths) then
		if plate then
			self:UpdateNameplate(plate.plateFrame, plate.plateData, plate.plateFrame:IsShown())
		end	

		-- Update player score
		if player.name == self.PLAYER_NAME then
			local rank
			if player.kills > 0 then
				rank = "Rank #" .. player.rank
			else
				rank = ""
			end
			
			score.fontString:SetText(player.kills .. " - " .. player.deaths .. " " .. rank)
			
			-- Highlight score frame
			if player.kills > originalPlayerKills and self.db.flashEnabled then
				LibStub("LibButtonGlow-1.0").ShowOverlayGlow(score)
				LibStub("LibButtonGlow-1.0").HideOverlayGlow(score)
			end
			
			if player.rank > 0 and player.rank <= self.MAX_RANK and player.kills > 0 then
				score.rankTex:SetTexture("Interface\\AddOns\\MongoMon\\Res\\Rank" .. player.rank)
				score.crestFrame:Show()
				score.rankFrame:Show()
			else
				score.crestFrame:Hide()
				score.rankFrame:Hide()
			end
			
			-- Assign our player object one time to the global reference
			if not self.player then
				self.player = player
			end
		end		
	end	
end

--[[ 
 -	Helper function to get the compelte target name.
 -	
 -	Gets the complete name of the currently selected target
 -	Returns: Name - Server
--]]
function MongoMon:getTargetName()

	local targetName, targetRealm = UnitName("target")
		
	-- So this is kind of crappy, players of the same realm will have a nil realm. 
	-- Check for realm and append to name for the love of stalin.
	if targetRealm then
		targetName = targetName .. "-" .. targetRealm
	end	
	
	return targetName
end

--[[
 -	Method modifies unit nameplates by adding the score of the unit
 -	above their nameplate in the format "Kills - Deaths" when criteria is met.
 -	
 -	WoW fonts: http://wowprogramming.com/utils/xmlbrowser/live/FrameXML/Fonts.xml
 -
 -	@param Table	- The frame object
 -	@param Table	- Related player data
 -	@param boolean	- Whether to hide or show the nameplate
--]]
function MongoMon:UpdateNameplate(plateFrame, plateData, hide)

	-- Validate parameters.
	if(not plateFrame or not plateData or not plateData.GUID or not plateData.name or plateData.type ~= "PLAYER" or not self.instancedPvp) then return end
	
	local name = self.guids[plateData.GUID]
	if not name then return end
	
	local player, plate = self.players[name], self.plates[name]
	
	if not player then -- Player not found
		return 
	elseif plate and not self.db.nameplatesEnabled then -- Nameplates disabled
		plate.fontString:Hide()
		plate.texture:Hide()
		self.plates[name] = nil
	elseif plate and hide then -- Hide existing plate
		plate.fontString:Hide()
		plate.texture:Hide()
		self.plates[name] = nil
	elseif plate and not hide and self.db.nameplatesEnabled then -- Update existing plate!
		plate.fontString:SetText(player.kills .. " - " .. player.deaths )
		if player.rank > 0 and player.rank <= self.MAX_RANK and player.kills > 0 then
			plate.texture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\Rank" .. player.rank)
			plate.texture:Show()
		else
			plate.texture:Hide()
		end
	elseif not plate and not hide and self.db.nameplatesEnabled then -- Add new plate!
		
		-- Create Fontstring for Kill - Death score
		local f = plateFrame:CreateFontString();
		
		f:SetFont(_G.NumberFont_Shadow_Small:GetFont(), 15, "THICKOUTLINE")
		if player.enemy then
			f:SetTextColor(1, 0.1, 0.1, 0.75)
		else
			f:SetTextColor(0, 1, 0, 1)
		end
		f:SetPoint("CENTER", 0, 20)
		f:SetText(player.kills .. " - " .. player.deaths)
		
		local t = plateFrame:CreateTexture()
		t:SetWidth(32)
		t:SetHeight(32)
		t:Hide()
		t:SetPoint("TOPLEFT", plateFrame, "TOPLEFT", 0, 15)

		if player.rank > 0 and player.rank <= self.MAX_RANK and player.kills > 0 then
			t:SetTexture("Interface\\AddOns\\MongoMon\\Res\\Rank" .. player.rank)
			t:Show()
		end

		-- Not sure if this is dangerous to keep references to data objects I don't understand.
		-- I do this because I keep everything keyed on player name as opposedto GUID since I don't
		-- know how to extract GUID from the battleground score data. I think the name plates lib
		-- uses GUID, as such they must have to parse combat logs or some such nonsense. /rant
		plate = {
			texture = t,				-- Texture according to rank, accomponies rankFont
			fontString = f,				-- Displays battleground k/d ratio
			plateData = plateData,		-- Other stuff
			plateFrame = plateFrame		-- Other stuff
		}
		self.plates[name] = plate
	end
end

--[[
 - Jesus christ this is a piece of shit. We need to listen to all the trash in the combat
 - log to get a handle on a player guid because fucking cocksucking GetBattlefieldScores 
 - doesn't supply it. Fucking piece of shit.
--]]
function MongoMon:CaptureGuids(sourceName, sourceGUID, sourceFlags, destName, destGUID, destFlags) 
	if not self.instancedPvp then return end
	
	if sourceGUID and sourceName and sourceFlags and bit.band(sourceFlags, 66888) then
		if not self.guids[sourceGUID] then
			self.guids[sourceGUID] = sourceName
		end
	end	
	
	if destGUID and destName and destFlags and bit.band(destFlags, 66888) then
		if destGUID ~= sourceGUID and destName ~= sourceName then
			if not self.guids[destGUID] then
				self.guids[destGUID] = destName
			end
		end
	end	
end

--[[
 -	Event handler for LibNameplateRegistry's LNR_ON_NEW_PLATE event.
 -	
 -	A new plate is in range. Let us do things.
--]]
function MongoMon:LNR_ON_NEW_PLATE(eventname, plateFrame, plateData)
	MongoMon:UpdateNameplate(plateFrame, plateData, false)
end

--[[
 -	Event handler for LibNameplateRegistry's LNR_ON_RECYCLE_PLATE event.
 -	
 -	I believe this means the player has gone out of range. Not sure if
 -	we care about that.
--]]
function MongoMon:LNR_ON_RECYCLE_PLATE(eventname, plateFrame, plateData)
	MongoMon:UpdateNameplate(plateFrame, plateData, true)
end

--[[
 -	Event handler for LibNameplateRegistry's LNR_ON_GUID_FOUND event.
 -
 -	Not sure why I care about GUIDs to be quite honest.
--]]
function MongoMon:LNR_ON_GUID_FOUND(eventname, frame, GUID, findmethod)
	local plateFrame, plateData = self:GetPlateByGUID(GUID)
	if plateFrame and plateData then
		self:UpdateNameplate(plateFrame, plateData, plateFrame:IsShown())
	end
end

--[[
 -	Event handler for LibNameplateRegistry's LNR_ERROR_FATAL_INCOMPATIBILITY event.
--]]
function MongoMon:LNR_ERROR_FATAL_INCOMPATIBILITY(eventname, icompatibilityType)
	-- Here you want to check if your add-on and LibNameplateRegistry are not
	-- outdated (old TOC). if they're both up to date then it means that
	-- another add-on author thinks his add-on is more important than yours. In
	-- this later case you can register LNR_ERROR_SETPARENT_ALERT and
	-- LNR_ERROR_SETSCRIPT_ALERT which will detect such behaviour and will give
	-- you the name of the incompatible add-on so you can inform your users properly
	-- about what's happening instead of just silently "not working".
	print(eventname, icompatibilityType)
end

--[[
 -	Main Addon event listener.
 -
 -	A few notes on RequestBattlefieldScoreData:
 -	
 -	Polls the WoW server for Battleground data. The response is
 -	asynchronous in the form of an UPDATE_BATTLEFIELD_SCORE event.
 -	
 -	The same behavior can be achieved by hitting ctrl + spacebar to manually
 -	inspect the bg data. This was making me so mad.
 -	
 -	So now I understand why kunda (BattleGroundTargets) does what he does
 -	with the PLAYER_REGEN_ENABLED event. This signifies the player is out of
 -	combat and happens at a regular enough frequency to capture the bg score data.	
--]]
local function EventHandler(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2 = select(1, ...)
		if damageEvent[type] then
			MongoMon:KillingBlowCheck(type, sourceName, destName, destFlags, select(12, ...))
		elseif type == "UNIT_DIED" then
			RequestBattlefieldScoreData()		
		end
		MongoMon:CaptureGuids(sourceName, sourceGUID, sourceFlags, destName, destGUID, destFlags)
	elseif event == "PLAYER_TARGET_CHANGED" then
		MongoMon:PlayerTargetChanged()
	elseif event == "PLAYER_REGEN_ENABLED" then
		RequestBattlefieldScoreData()
	elseif event == "UPDATE_BATTLEFIELD_SCORE" then
		MongoMon:UpdateBattlefieldScore()
	elseif event == "PLAYER_ENTERING_WORLD" then
		MongoMon:Initialize()
		RequestBattlefieldScoreData()
	end
end

-- Always listening
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", EventHandler)